--- helper to drop SAHARA_APP table and Procedure

IF EXISTS (select * from sys.sysprocedure sp key join sysusers u
 where sp.proc_name = 'SP_DROP_' and u.name = 'SAHARA_APP' ) THEN
  DROP PROCEDURE SAHARA_APP.SP_DROP_;
END IF;


CREATE PROCEDURE SAHARA_APP.SP_DROP_( IN @pObjectType LONG VARCHAR,
                                      IN @pObjectName LONG VARCHAR)
BEGIN
  DECLARE @ct UNSIGNED INTEGER;

  BEGIN

    SET @ct = 0;
    
    IF @pObjectType = 'TABLE' THEN 

        select count(1) into @ct from systable t key join sysusers u
        where t.table_name = @pObjectName
        and u.name = 'SAHARA_APP';
       
    END IF;
    
    IF @pObjectType = 'PROCEDURE' THEN 
          select count(1) into @ct  from sys.sysprocedure sp key join sysusers u
          where sp.proc_name = @pObjectName and u.name = 'SAHARA_APP'

    END IF;

    IF @ct > 0 THEN
        execute immediate 'drop ' || @pObjectType || ' SAHARA_APP."' || @pObjectName || '"';
    END IF;

  END;
END;


------------------------------ Staging table for business view overview information

CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_OVERVIEW_');
CREATE TABLE SAHARA_APP.IE_OVERVIEW_
(
    BVIEW_NAME varchar(12) NOT NULL,
    DISPLAY_NAME varchar(64) NOT NULL,
    NOTE LONG VARCHAR
);


comment on table SAHARA_APP.IE_OVERVIEW_ is
'Import/Export staging table for view overview information. Contains the
Oracle friendly view name, a user-friendly display name, and any comments.
Views are imported one at a time. Values in this table represent the
view that is currently being imported.';

comment on column SAHARA_APP.IE_OVERVIEW_.BVIEW_NAME is
'Oracle friendly name for this business view, constrained to 12 characters.';

comment on column SAHARA_APP.IE_OVERVIEW_.DISPLAY_NAME is
'User friendly display name for this business view.';

comment on column SAHARA_APP.IE_OVERVIEW_.NOTE is
'Any comments or notes for this business view.';

--
-- staging table for types supported by the business view.
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_TYPES_');
CREATE TABLE SAHARA_APP.IE_TYPES_
(
    TYPE_NAME VARCHAR(26) NOT NULL
);

comment on table SAHARA_APP.IE_TYPES_ is
'Import/Export staging table containing the types supported by this view. ';

comment on column SAHARA_APP.IE_TYPES_.TYPE_NAME is
'Each row in this column represents a type supported by this business view.
Supported types are defined in the SAHARA.BVIEW_DIMENSION table.';

-- support only types defined in the SAHARA.BVIEW_DIMENSION table.
alter table SAHARA_APP.IE_TYPES_
   add constraint FK_IETYPES_DIMESION
   foreign key (TYPE_NAME)
   references SAHARA.BVIEW_DIMENSION(DIMENSION);

--
-- staging table for the business view levels.
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_LEVELS_');
CREATE TABLE SAHARA_APP.IE_LEVELS_
(
    DEPTH NUMERIC NOT NULL,
    LEVEL_NAME VARCHAR(30) NOT NULL,
    LEVEL_DISPLAY_NAME VARCHAR(256) NOT NULL,
    NOTE LONG VARCHAR DEFAULT NULL
);

comment on table SAHARA_APP.IE_LEVELS_ is
'Import/Export staging table containing the business view level hierarchy.
The top of the hierarchy is the first row, the bottom is the last row.
This hierarchy imposes an ordering relationship on the objects in this business view.';

comment on column SAHARA_APP.IE_LEVELS_.DEPTH is
'Depth of this level. The root of the hierarchy is at 0, the first child of root is 1, etc';

comment on column SAHARA_APP.IE_LEVELS_.LEVEL_NAME is
'Oracle friendly name for this level, constrained to 30 characters.';

comment on column SAHARA_APP.IE_LEVELS_.LEVEL_DISPLAY_NAME is
'User friendly display name for this level.';

comment on column SAHARA_APP.IE_LEVELS_.NOTE is
'Any comments or notes for this level.';

--
-- Import status table to record the status of each import.
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_STATUS');
CREATE TABLE SAHARA_APP.IE_STATUS
(
    IMPORT_TIME timestamp default CURRENT UTC TIMESTAMP,
    SESSION_KEY NUMERIC(10) NOT NULL,
    VIEWNAME VARCHAR(256) NOT NULL,
    STATUS_CODE VARCHAR(256) NOT NULL
);

grant SELECT,INSERT,DELETE,update on SAHARA_APP.IE_STATUS to SAHARA;

alter table SAHARA_APP.IE_STATUS
   add constraint FK_IE_STATUS_SESSION foreign key (SESSION_KEY)
      references SAHARA."SESSION" (SESSION_KEY)
      on delete cascade;

comment on table SAHARA_APP.IE_STATUS is
'Import status table to record the status of each import. Every import
attempt is given a session key. Use this key to query for the results
of a specific import/export request.';

comment on column SAHARA_APP.IE_STATUS.IMPORT_TIME is
'Time the import request completed.';

comment on column SAHARA_APP.IE_STATUS.SESSION_KEY is
'The session key that identifies each import request.';

comment on column SAHARA_APP.IE_STATUS.VIEWNAME is
'The basename for all the files in the import set. e.g. Geography.';

comment on column SAHARA_APP.IE_STATUS.STATUS_CODE is
'A free form status code indicating the final status of the import.';

--
-- staging table for host objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_HOST_');

CREATE TABLE SAHARA_APP.IE_HOST_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    HOST_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);
comment on table SAHARA_APP.IE_HOST_ is
'Import/Export staging table for host objects.';

comment on column SAHARA_APP.IE_HOST_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_HOST_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_HOST_.HOST_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_HOST_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_HOST_.IP is
'The IP address for the object';

comment on column SAHARA_APP.IE_HOST_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_HOST_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_HOST_
    add constraint C_IEHOST_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for array objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_ARRAY_');
CREATE TABLE SAHARA_APP.IE_ARRAY_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    ARRAY_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_ARRAY_ is
'Import/Export staging table for array objects.';

comment on column SAHARA_APP.IE_ARRAY_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_ARRAY_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_ARRAY_.ARRAY_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_ARRAY_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_ARRAY_.IP is
'The IP address for the object';

comment on column SAHARA_APP.IE_ARRAY_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_ARRAY_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_ARRAY_
    add constraint C_IEARRAY_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for switch objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_SWITCH_');
CREATE TABLE SAHARA_APP.IE_SWITCH_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    SWITCH_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_SWITCH_ is
'Import/Export staging table for switch objects.';

comment on column SAHARA_APP.IE_SWITCH_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_SWITCH_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_SWITCH_.SWITCH_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_SWITCH_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_SWITCH_.IP is
'The IP address for the object';

comment on column SAHARA_APP.IE_SWITCH_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_SWITCH_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_SWITCH_
    add constraint C_IESWITCH_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for app_group objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_APP_GROUP_');
CREATE TABLE SAHARA_APP.IE_APP_GROUP_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    APP_GROUP_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256) DEFAULT NULL,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_APP_GROUP_ is
'Import/Export staging table for application group objects.';

comment on column SAHARA_APP.IE_APP_GROUP_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_APP_GROUP_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_APP_GROUP_.APP_GROUP_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_APP_GROUP_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_APP_GROUP_.IP is
'IP address - will always be null for app_groups. We include this
here to make the correlation procedure more readable.';

comment on column SAHARA_APP.IE_APP_GROUP_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_APP_GROUP_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_APP_GROUP_
    add constraint C_IEAPPGROUP_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp volume objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NETAPP_VOLUME_');
create table SAHARA_APP.IE_NETAPP_VOLUME_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NETAPP_VOLUME_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    PATH VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_NETAPP_VOLUME_ is
'Import/Export staging table for NetApp volume objects.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.NETAPP_VOLUME_KEY is
'The shara.netapp_volume object unique key for this object.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.FILER_NAME is
'The name of the owner of this volume';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.PATH is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the volume.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NETAPP_VOLUME_
    add constraint C_IE_NAVOLUME_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp share types
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NETAPP_SHARE_');
create table SAHARA_APP.IE_NETAPP_SHARE_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NETAPP_SHARE_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    PATH VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_NETAPP_SHARE_ is
'Import/Export staging table for NetApp share objects.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.NETAPP_SHARE_KEY is
'The shara.netapp_share object unique key for this object.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.FILER_NAME is
'The name of the owner of this share.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.PATH is
'The fully qualified path for this share. This combined with the filer
name forms a unique identifier for the share.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NETAPP_SHARE_
    add constraint C_IE_NASHARE_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp qtree objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NETAPP_QTREE_');
create table SAHARA_APP.IE_NETAPP_QTREE_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NETAPP_QTREE_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    PATH VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_NETAPP_QTREE_ is
'Import/Export staging table for NetApp qtree objects.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.NETAPP_QTREE_KEY is
'The shara.netapp_qtree object unique key for this object.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.FILER_NAME is
'The name of the owner of this qtree.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.PATH is
'The fully qualified path for this qtree. This combined with the filer
name forms a unique identifier for the qtree.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NETAPP_QTREE_
    add constraint C_IE_NAQTREE_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp volume objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NAS_FILESYSTEM_');
create table SAHARA_APP.IE_NAS_FILESYSTEM_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NAS_FILESYSTEM_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    NAS_FILESYSTEM_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_NAS_FILESYSTEM_ is
'Import/Export staging table for NAS filesystem objects.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.NAS_FILESYSTEM_KEY is
'The shara.netapp_volume object unique key for this object.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.FILER_NAME is
'The name of the owner of this filesystem';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.NAS_FILESYSTEM_NAME is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the filesystem.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NAS_FILESYSTEM_
    add constraint C_IE_NASFS_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for Fileystem objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_FILESYSTEM_');
create table SAHARA_APP.IE_FILESYSTEM_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    FILESYSTEM_KEY NUMERIC(10),
    HOST_NAME VARCHAR(256),
    FILESYSTEM_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_FILESYSTEM_ is
'Import/Export staging table for filesystem objects.';

comment on column SAHARA_APP.IE_FILESYSTEM_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_FILESYSTEM_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_FILESYSTEM_.FILESYSTEM_KEY is
'The shara.filesystem object unique key for this object.';

comment on column SAHARA_APP.IE_FILESYSTEM_.HOST_NAME is
'The name of the owner of this filesystem';

comment on column SAHARA_APP.IE_FILESYSTEM_.FILESYSTEM_NAME is
'The fully qualified path for this filesystem. This combined with the filer
name forms a unique identifier for the filesystem.';

comment on column SAHARA_APP.IE_FILESYSTEM_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_FILESYSTEM_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_FILESYSTEM_
    add constraint C_IE_FS_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for Volume objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_VOLUME_');
create table SAHARA_APP.IE_VOLUME_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    VOLUME_KEY NUMERIC(10),
    HOST_NAME VARCHAR(256),
    VOLUME_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_VOLUME_ is
'Import/Export staging table for volume objects.';

comment on column SAHARA_APP.IE_VOLUME_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_VOLUME_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_VOLUME_.VOLUME_KEY is
'The shara.volume object unique key for this object.';

comment on column SAHARA_APP.IE_VOLUME_.HOST_NAME is
'The name of the owner of this volume';

comment on column SAHARA_APP.IE_VOLUME_.VOLUME_NAME is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the volume.';

comment on column SAHARA_APP.IE_VOLUME_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_VOLUME_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_VOLUME_
    add constraint C_IE_VOLUME_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for Diskgroup objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_DISKGROUP_');
create table SAHARA_APP.IE_DISKGROUP_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    DISKGROUP_KEY NUMERIC(10),
    HOST_NAME VARCHAR(256),
    DISKGROUP_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_DISKGROUP_ is
'Import/Export staging table for diskgroup objects.';

comment on column SAHARA_APP.IE_DISKGROUP_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_DISKGROUP_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_DISKGROUP_.DISKGROUP_KEY is
'The sahara.diskgroup object unique key for this object.';

comment on column SAHARA_APP.IE_DISKGROUP_.HOST_NAME is
'The name of the owner of this diskgroup';

comment on column SAHARA_APP.IE_DISKGROUP_.DISKGROUP_NAME is
'The fully qualified name for this diskgroup.';

comment on column SAHARA_APP.IE_DISKGROUP_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_DISKGROUP_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_DISKGROUP_
    add constraint C_IE_DG_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for ServiceGroup objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_SERVICEGROUP_');
create table SAHARA_APP.IE_SERVICEGROUP_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    SERVICEGROUP_KEY NUMERIC(10),
    CLUSTER_NAME VARCHAR(256),
    SERVICEGROUP_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_SERVICEGROUP_ is
'Import/Export staging table for service group objects.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.SERVICEGROUP_KEY is
'The shara.servicegroup object unique key for this object.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.CLUSTER_NAME is
'The name of the cluster of this service group';

comment on column SAHARA_APP.IE_SERVICEGROUP_.SERVICEGROUP_NAME is
'The name of service group.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_SERVICEGROUP_
    add constraint C_IE_SG_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for cluster objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_CLUSTER_');

CREATE TABLE SAHARA_APP.IE_CLUSTER_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    CLUSTER_KEY NUMERIC(10),
    NAME VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);
comment on table SAHARA_APP.IE_CLUSTER_ is
'Import/Export staging table for cluster objects.';

comment on column SAHARA_APP.IE_CLUSTER_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_CLUSTER_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_CLUSTER_.CLUSTER_KEY is
'The shara.cluster object unique key for this object.';

comment on column SAHARA_APP.IE_CLUSTER_.NAME is
'The cluster name for the object';

comment on column SAHARA_APP.IE_CLUSTER_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_CLUSTER_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_CLUSTER_
    add constraint C_IECLUSTER_REJECTED check(REJECTED in ('T', 'F'));

--
-- Staging table for business view node information.  This holds an entry for
-- each node in a business view tree and a NODE_ID generated by the import
-- script.  It also generates a PARENT_ID for each row.  These IDs are used
-- to populate the BVIEW_NODE_KEY and PARENT_BVIEW_NODE_KEY after the rows
-- are staged and the nodes added to the production table.  These same ids
-- are then used to associate the node with each assigned object.
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NODE_');
create table SAHARA_APP.IE_NODE_
(
    DISPLAY_NAME VARCHAR(254) not null,
    DEPTH NUMERIC(3,0) not null,
    NODE_ID NUMERIC(10),
    PARENT_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    PARENT_BVIEW_NODE_KEY NUMERIC(10)
);

comment on column SAHARA_APP.IE_NODE_.DISPLAY_NAME is
'The display name for the business view node.';

comment on column SAHARA_APP.IE_NODE_.DEPTH is
'The depth of this node in the hierarchy. The root of the node is at depth 0,
 the next depth is 1, etc.';

comment on column SAHARA_APP.IE_NODE_.NODE_ID is
'Identifier generated by the import script to uniquely identify this node.';

comment on column SAHARA_APP.IE_NODE_.PARENT_ID is
'Identifier of this node parent generated by the import script to uniquely identify
the parent of this node. If the node is at the top of the hierarchy this column
will be null.';

comment on column SAHARA_APP.IE_NODE_.BVIEW_NODE_KEY is
'The key assigned by the SAHARA.BVIEW_API.AddNode procedure. This is the id the node
is identified by in ER.';

comment on column SAHARA_APP.IE_NODE_.PARENT_BVIEW_NODE_KEY is
'The BVIEW_NODE_KEY for the parent node if this node.';





CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_StringToSet');

CREATE PROCEDURE SAHARA_APP.SP_IE_StringToSet
(IN @pString LONG VARCHAR)
RESULT (StringSet VARCHAR(255))
BEGIN
  DECLARE @vStart INTEGER;
  DECLARE @vValue VARCHAR(255);
  DECLARE @incr INTEGER;
  
  BEGIN   
    
    SET @vStart = 1;
    SET @incr = 1;
    
    BEGIN
      
      WHILE (@vStart <> 0 )
      LOOP
      
	SET @pstring = SUBSTRING(@pstring,@vStart);
        SET @vStart = CHARINDEX(',',@pstring);
        IF @vStart <> 0 THEN 
          SET @vValue = SUBSTRING(trim(SUBSTRING(@pstring, 1, @vStart - 1)), 1, 256);
          SET @vStart = @vStart + 1;
        ELSE 
          SET @vValue = trim(@pstring);

        END IF; 
        IF @incr = 1 THEN 
          SELECT @vValue as vValue INTO #vset ;
        ELSE 
	  INSERT INTO #vset(vValue) VALUES (@vValue);
	END IF;   
        SET @incr = 0;                  
      
      END LOOP;
      
      SELECT vValue AS StringSet FROM #vset;

      END;
  
  END;

END;

CALL SAHARA_APP.SP_DROP_('PROCEDURE','FN_IE_StringSetsOverlap');

CREATE FUNCTION SAHARA_APP.FN_IE_StringSetsOverlap
(
  IN @pString1 LONG VARCHAR,
  IN @pString2 LONG VARCHAR
)
  RETURNS BIT
BEGIN
   DECLARE @IsOverLap BIT;

   BEGIN 

    -- convert the strings to sets
    
     SELECT StringSet INTO #vSet1 FROM SAHARA_APP.SP_IE_StringToSet(@pString1);
     SELECT StringSet INTO #vSet2 FROM SAHARA_APP.SP_IE_StringToSet(@pString2);
    
    -- if either set is empty, there's no overlap

     IF ( SELECT ISNULL(COUNT(1),0) FROM #vSet1 ) = 0  THEN
       RETURN 0;
     END IF;

     IF  ( SELECT ISNULL(COUNT(1),0) FROM #vSet2 ) = 0  THEN
        RETURN 0;
     END IF;

    -- iterate the smaller set and test each of its members for inclusion
    -- in the larger set

     SET @IsOverLap = 0; 
   
     SELECT ISNULL(MAX(1),0) INTO @IsOverLap FROM #vSet1 Set1,  #vSet2 Set2
     WHERE Set1.StringSet = Set2.StringSet;
   
     RETURN @IsOverLap;
  
  END;
      
END;

CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateImportedViews');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateImportedViews
(IN @pOverWrite BIT)
BEGIN
  DECLARE @vViewName VARCHAR(12);
  DECLARE @vViewDisplayName VARCHAR(64);
  DECLARE @vViewNote LONG VARCHAR;
  DECLARE @vViewKey INTEGER;
  DECLARE @vLevelKey INTEGER;
  DECLARE VIEW_EXISTS EXCEPTION FOR
        SQLSTATE '42000'; 

  BEGIN

    
    -- extract the view information from the staging tables

    select 
      BVIEW_NAME, DISPLAY_NAME, NOTE
      into @vViewName, @vViewDisplayName, @vViewNote
    from 
      SAHARA_APP.IE_OVERVIEW_;

    -- See if the view already exists
    BEGIN
       -- raise an exception if we're not over-writing

       
      SET @vViewKey = 0;
      
      select 
        ISNULL(BVIEW_KEY,0) into @vViewKey
      from 
        SAHARA.BVIEW_IMPL
      where 
        NAME = @vViewName;

      IF  @vViewKey <> 0 THEN
        --- If exixsts and overwrite is not set then send it back.
            IF @pOverWrite = 0  THEN
                   SIGNAL VIEW_EXISTS;
            END IF;


        CALL SP_BVIEW_Delete1(@vViewKey);

      END IF;
      
      SET @vViewKey = SAHARA.FN_BVIEW_Create1(@vViewDisplayName, @vViewName, @vViewNote);
    
    END;

    -- cursor through the level staging table in depth order and create our levels
    
    FOR LevelRecLOOP AS LevelRecCUR CURSOR FOR 
      SELECT 
        LEVEL_NAME AS LEVEL_NAME_R, 
        LEVEL_DISPLAY_NAME AS LEVEL_DISPLAY_NAME_R, 
	NOTE AS NOTE_R
      FROM 
        SAHARA_APP.IE_LEVELS_
      ORDER BY DEPTH
    DO
       -- insert view level rows.

       SET @vLevelKey = SAHARA.FN_BVIEW_AddLevel(@vViewKey,
                                          LEVEL_DISPLAY_NAME_R,
                                          LEVEL_NAME_R,
                                          NOTE_R);
    END FOR;

    -- add dimension support for each
    
    FOR IETypesLOOP AS IETypesCUR CURSOR FOR 
      select 
        TYPE_NAME AS TYPE_NAME_R
      from 
        SAHARA_APP.IE_TYPES_
    DO
    
	CALL SAHARA.SP_BVIEW_AddDimensionSupport(@vViewKey, TYPE_NAME_R);

    END FOR;

  END;

END;


CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateNodesAtLevel');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateNodesAtLevel(IN @pViewName LONG VARCHAR,
                                                    IN @pLevelDepth INTEGER,
                                                    IN @pSessionKey INTEGER)
BEGIN
  DECLARE @vRootKey INTEGER;
  DECLARE @v_BVIEW_NODE_KEY numeric(10,0);
  DECLARE @v_parent_BVIEW_NODE_KEY numeric(10,0);
  DECLARE @FUNC varchar(64);
  BEGIN

    SET @vRootKey = 0;


   set @FUNC = 'SP_IE_CorrelateNodesAtLevel';
    
    call SAHARA.SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');


   -- get the node key for the root of the view
   
    select 
      ISNULL(BVIEW_KEY,0) into @vRootKey
    from 
      SAHARA.BVIEW
    where 
      NAME = @pViewName;

    IF @vRootKey = 0 THEN

      CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewNotFound,
            'SP_IE_CorrelateNodesAtLevel',
            'the business view not found ' || @pViewName
        );

    END IF; 

  END;
   
  FOR IETypesLOOP AS IETypesCUR CURSOR FOR 
    select 
       DISPLAY_NAME AS DISPLAY_NAME_R,
       NODE_ID AS NODE_ID_R,
       PARENT_ID AS PARENT_ID_R,
       BVIEW_NODE_KEY AS BVIEW_NODE_KEY_R,
       PARENT_BVIEW_NODE_KEY AS PARENT_BVIEW_NODE_KEY_R
    from 
      SAHARA_APP.IE_NODE_
    where 
      DEPTH = @pLevelDepth
    for update of BVIEW_NODE_KEY,PARENT_BVIEW_NODE_KEY

    DO

      IF parent_BVIEW_NODE_KEY_R IS NULL THEN

        SET  @v_parent_BVIEW_NODE_KEY = @vRootKey;
      
      ELSE
      
	SET  @v_parent_BVIEW_NODE_KEY = parent_BVIEW_NODE_KEY_R;
      
      END IF;

      SET @v_BVIEW_NODE_KEY = BVIEW_NODE_KEY_R;

      IF @v_BVIEW_NODE_KEY is NULL THEN
      
        BEGIN

          SET @v_BVIEW_NODE_KEY = SAHARA.FN_BVIEW_AddNode(@v_parent_BVIEW_NODE_KEY,
                                                          DISPLAY_NAME_R,
                                                          NULL);
         EXCEPTION WHEN OTHERS THEN 
          
		IF SQLCODE = -196 THEN 

                   select BVIEW_NODE_KEY into @v_BVIEW_NODE_KEY
                      from SAHARA.BVIEW_NODE
                     where PARENT_BVIEW_NODE_KEY =@v_parent_BVIEW_NODE_KEY
                       and DISPLAY_NAME = DISPLAY_NAME_R;

                    -- TODO : discard or update based on options

               END IF;
       END;
     
      END IF;

   -- update the current node
      
      update 
        SAHARA_APP.IE_NODE_
      set 
        BVIEW_NODE_KEY= @v_BVIEW_NODE_KEY,
        PARENT_BVIEW_NODE_KEY= @v_parent_BVIEW_NODE_KEY
      where 
       DISPLAY_NAME = DISPLAY_NAME_R AND 
       NODE_ID = NODE_ID_R AND 
       isnull(PARENT_ID_R,0)  in (PARENT_ID,0)   AND
       isnull(BVIEW_NODE_KEY_R,0) in (BVIEW_NODE_KEY,0) AND
       isnull(PARENT_BVIEW_NODE_KEY_R,0) in (PARENT_BVIEW_NODE_KEY,0);

      
   -- set the parent node key for all the direct children of this node

      update 
        SAHARA_APP.IE_NODE_
      set 
        PARENT_BVIEW_NODE_KEY= @v_BVIEW_NODE_KEY
      where 
        PARENT_ID= NODE_ID_R;
    
    END FOR;
   
END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjectByObjectName');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjectByObjectName(IN @pType  VARCHAR(255))
BEGIN
  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  BEGIN 

    SET @vSQL_TEMPLATE ='begin FOR StageCursorLOOP AS StageCursor CURSOR FOR select HOST_NAME AS HOST_NAME_R, $TYPE$_NAME AS $TYPE$_NAME_R '||
                        ' from SAHARA_APP.IE_$TYPE$_ where HOST_NAME is not null ' ||
                   'and $TYPE$_NAME is not null and $TYPE$_KEY is null and BVIEW_NODE_KEY is not null for update of $TYPE$_KEY ' ||
                   'DO ' ||
                'update SAHARA_APP.IE_$TYPE$_ ' ||
                    'set $TYPE$_KEY = ' ||
                        '(select FIRST $TYPE$_KEY ' ||
                           'from SAHARA.$TYPE$_CURRENT OT ' ||
                           'join SAHARA.HOST_CURRENT ON HOST_CURRENT.HOST_KEY = OT.HOST_KEY ' ||
                          'where OT.$PATH_COLUMN$ = $PATH_VALUE$ ' ||
                            'and HOST_CURRENT.DISPLAY_NAME = HOST_NAME_R ' ||
                            ') '|| 
	          'WHERE HOST_NAME = HOST_NAME_R '||
		  ' AND $TYPE$_NAME = $TYPE$_NAME_R ; ' ||
            'end for; ' ||
        'end;';
    SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);
   
    IF @pType = 'FILESYSTEM' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'DISPLAY_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'FILESYSTEM_NAME_R');
    END IF;
    
    IF @pType = 'VOLUME' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'VOLUME_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'VOLUME_NAME_R');
    END IF;

    IF @pType = 'DISKGROUP' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'DISPLAY_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'DISKGROUP_NAME_R');
    END IF;

 EXECUTE IMMEDIATE @vSQL;
  END;
END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjects');
CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjects
(
IN @viewname LONG VARCHAR,
IN @session_key numeric(10)
)
BEGIN
    DECLARE @vRootKey NUMERIC(10);
    DECLARE @vStagingTable VARCHAR(61);
    DECLARE @vType VARCHAR(30);
    DECLARE @vSQL LONG VARCHAR;
   DECLARE @FUNC varchar(64);
    
    BEGIN
    -- get the node key for the root of the view
     SET @vRootKey = 0 ; 

    set @FUNC = 'SP_IE_CorrelateObjects';
    
    call SAHARA.SP_Log_LogDebug(@session_key, @FUNC, 'entry');


     select BVIEW_KEY
      into @vRootKey
      from SAHARA.BVIEW
     where NAME = @viewname;

    if @vRootKey =0  then
      
      CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewNotFound,
            'SP_IE_CorrelateObjects',
            'the business view not found ' || @viewname
        );

    end if;

    -- Iterate over the supported types.  For each, join the node and object
    -- staging tables and set the BVIEW_NODE_KEY column in the object staging
    -- table.  Use this join to call AssociateDimensionRows.  Select all the
    -- objects that don't have BVIEW_NODE_KEY values and mark them
    -- as rejected.




    FOR IETypesLOOP AS IETypesCUR CURSOR FOR 
    select TYPE_NAME from SAHARA_APP.IE_TYPES_
    
    DO
      SET @vType = UPPER(TYPE_NAME);
      SET @vStagingTable = 'SAHARA_APP.IE_' || @vType || '_';

        -- Set the business view node keys for the objects based on
        -- the IE node IDs previously populated by CorrelateNodesAtLevel.

        SET @vSQL =
            'update ' || @vStagingTable || ' STAGE ' ||
               'set BVIEW_NODE_KEY = ' ||
                    '(select BVIEW_NODE_KEY ' ||
                       'from SAHARA_APP.IE_NODE_ ' ||
                      'where STAGE.NODE_ID = IE_NODE_.NODE_ID)';
        execute immediate @vSQL;

        -- Now do the object correlation by filling in the <type>_KEY column
        -- of the object staging table.  If multiple object records match
        -- a criteria, associate the node with first object one we find.

        -- First we'll try based on the object's OID.
        SET @vSQL =
            'update ' || @vStagingTable || ' STAGE ' ||
               'set ' || @vType || '_KEY = ' ||
                    '(select ' || @vType || '_KEY ' ||
                       'from SAHARA.' || @vType || '_CURRENT OBJECT_TYPE ' ||
                      'where OBJECT_TYPE.' || @vType || '_OID = STAGE.OBJECT_UNIQUE_KEY) ' ||
             'where ' || @vType || '_KEY is null ' ||
               'and BVIEW_NODE_KEY is not null';
        execute immediate @vSQL;

        -- try matching the IP address for object types that have them
  
	if @vType in ('HOST', 'ARRAY', 'SWITCH') then
            CALL SAHARA_APP.SP_IE_CorrelateObjectsByIPAddress(@vType);
        end if;

        -- try matching the filer name and fully qualified path
        -- for netapp objects

        if @vType in ('NETAPP_VOLUME', 'NETAPP_SHARE', 'NETAPP_QTREE') then
            CALL SP_IE_CorrelateObjectByNetAppPath(@vType);
        end if;

		-- try matching the host name and object display name
        -- for filesystem, volume and diskgroup objects
        if @vType in ('FILESYSTEM', 'VOLUME', 'DISKGROUP') then
            CALL SP_IE_CorrelateObjectByObjectName(@vType);
        end if;

        -- match against the object name

        if @vType in ('APP_GROUP', 'HOST', 'ARRAY', 'SWITCH') then
            SET @vSQL =
                'update ' || @vStagingTable || ' STAGE ' ||
                    'set ' || @vType || '_KEY = ' ||
                        '(select FIRST ' || @vType || '_KEY ' ||
                            'from SAHARA.' || @vType || '_CURRENT OT ' ||
                           'where OT.DISPLAY_NAME = STAGE."NAME" ' ||
                             ') ' ||
                    'where ' || @vType || '_KEY is null ' ||
                      'and BVIEW_NODE_KEY is not null';
            execute immediate @vSQL;
        end if;

        -- Object corellation is complete and recorded in the staging tables.
        -- Create the node to object associations.

        CALL SP_IE_CreateNodeObjectAssociations(@vType);

        -- Anything that correlated is NOT rejected.  Everything else is
        -- already marked rejected.

        SET @vSQL = 'update ' || @vStagingTable || ' ' ||
                   'set REJECTED = ''F'' ' ||
                 'where ' || @vType || '_KEY is not null ' ||
                   'and BVIEW_NODE_KEY is not null';
        execute immediate @vSQL;

	-- Anything that is NOT correlated is rejected.  Everything else is
        -- already marked NOT rejected.

        SET @vSQL = 'update ' || @vStagingTable || ' ' ||
                   'set REJECTED = ''T'' ' ||
                 'where ' || @vType || '_KEY is null ' ||
                   'and BVIEW_NODE_KEY is not null';
   
	execute immediate @vSQL;

        CALL SAHARA.SP_BVIEW_AddDimensionSupport(@vRootKey, @vType);

        
    end for;

    call SAHARA.SP_BVIEW_PublishBV();
  END ;

END;


CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjectsByIPAddress');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjectsByIPAddress(IN @pType  varchar(255))
BEGIN
  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  BEGIN
    
    SET @vSQL_TEMPLATE = 'begin ' ||
            'FOR StageCursorLOOP AS StageCursor CURSOR FOR ' ||
	    'select IP AS IP_R ' ||
                  'from SAHARA_APP.IE_$TYPE$_ ' ||
                 'where IP is not null ' ||
                   'and $TYPE$_KEY is null ' ||
                   'and BVIEW_NODE_KEY is not null ' ||
                'for update OF $TYPE$_KEY ' ||
            'DO ' ||
                'update SAHARA_APP.IE_$TYPE$_ ' ||
                    'set $TYPE$_KEY = ' ||
                        '(select FIRST $TYPE$_KEY ' ||
                           'from SAHARA.$TYPE$_CURRENT ' ||
                          'where 1 = SAHARA_APP.'||'FN_IE_StringSetsOverlap($IP_COLUMN$, IP_R) ' ||
                            'and $IP_COLUMN$ <> ''UNKNOWN'' ' ||
                            ') ' ||
                 'where IP = IP_R; ' ||
            'END FOR; ' ||
        'end; ';

    SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);

    if @pType in ('ARRAY', 'SWITCH') then
        SET @vSQL = replace(@vSQL, '$IP_COLUMN$', 'SNMP_IP_ADDRESS');
    else
        SET @vSQL = replace(@vSQL, '$IP_COLUMN$', 'IP_ADDRESS');
    end if;

    EXECUTE IMMEDIATE @vSQL;

  END;
END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CreateNodeObjectAssociations');

CREATE PROCEDURE SAHARA_APP.SP_IE_CreateNodeObjectAssociations(IN @pType LONG VARCHAR)
BEGIN

  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vKeys LONG VARCHAR;

  BEGIN 
    SET @vSQL_TEMPLATE = 'begin ' ||
                         'FOR StageCursorLOOP AS StageCursor CURSOR FOR ' ||
                         'select distinct BVIEW_NODE_KEY as BVIEW_NODE_KEY_R  ' ||
                         'from SAHARA_APP.IE_$TYPE$_ ' ||
                         'where BVIEW_NODE_KEY is not null ' ||
            'do ' ||
                'delete from sahara.DimensionRowKeyTable; INSERT INTO SAHARA.DimensionRowKeyTable select $TYPE$_KEY ' ||
                  ' from SAHARA_APP.IE_$TYPE$_ ' ||
                 'where BVIEW_NODE_KEY = BVIEW_NODE_KEY_R ' ||
                   'and $TYPE$_KEY is not null; ' ||
               
                  ' CALL SAHARA.SP_BVIEW_AssociateDimensionRows(BVIEW_NODE_KEY_R, ''$TYPE$''); ' ||

            'end for; ' ||
        'end;';
   

    BEGIN
      SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);
      EXECUTE IMMEDIATE @vSQL;
    end;
  END;

END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjectByNetAppPath');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjectByNetAppPath(IN @pType LONG VARCHAR)
BEGIN
  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  
  BEGIN
    SET @vSQL_TEMPLATE ='begin ' ||
                        'FOR StageCursorLOOP AS StageCursor CURSOR FOR ' ||
                'select FILER_NAME AS FILER_NAME_R, PATH AS PATH_R ' ||
                  'from SAHARA_APP.IE_$TYPE$_ ' ||
                 'where FILER_NAME is not null ' ||
                   'and PATH is not null ' ||
                   'and $TYPE$_KEY is null ' ||
                   'and BVIEW_NODE_KEY is not null ' ||
                'for update ' || ' DO ' ||
                'update SAHARA_APP.IE_$TYPE$_ ' ||
                    'set $TYPE$_KEY = ' ||
                        '(select FIRST $TYPE$_KEY ' ||
                           'from SAHARA.$TYPE$_CURRENT OT ' ||
                           'join SAHARA.ARRAY_CURRENT ON  ARRAY_CURRENT.ARRAY_KEY =  OT.ARRAY_KEY ' ||
                          'where OT.$PATH_COLUMN$ = $PATH_VALUE$ ' ||
                            'and ARRAY_CURRENT.DISPLAY_NAME = FILER_NAME_R ' ||
                            ') ' ||
                 ' WHERE  FILER_NAME = FILER_NAME_R AND PATH = PATH_R;' ||
            'end FOR; ' ||
        'end;';

    SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);

    case @pType

        when 'NETAPP_VOLUME' then
          SET @vSQL = replace(@vSQL, '$PATH_COLUMN$', 'DISPLAY_NAME');
            -- Volume paths are prefixed with '/vol/'.  We need to strip these
            -- out when actually comparing against the _CURRENT table.
          SET @vSQL = replace(@vSQL, '$PATH_VALUE$', 'substr(PATH_R, 6)');

        when 'NETAPP_SHARE' then

            SET @vSQL = replace(@vSQL, '$PATH_COLUMN$', 'SHARE_PATH');
            SET @vSQL = replace(@vSQL, '$PATH_VALUE$', 'PATH_R');


        when 'NETAPP_QTREE' then

            SET @vSQL = replace(@vSQL, '$PATH_COLUMN$', 'QTREE_PATH');
            SET @vSQL = replace(@vSQL, '$PATH_VALUE$', 'PATH_R');
    end case;

    EXECUTE IMMEDIATE @vSQL;
END;

END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateNodesAndObjects');
CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateNodesAndObjects
(
IN @pViewName VARCHAR(12),
IN @pSessionKey NUMERIC(10)
)
BEGIN
  DECLARE @vRootKey NUMERIC(10);
  DECLARE @FUNC varchar(64);
  BEGIN
    SET @vRootKey = 0 ;

    set @FUNC = 'SP_IE_CorrelateNodesAndObjects';
    
    call sahara.SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

      select BVIEW_KEY into @vRootKey
        from SAHARA.BVIEW
       where name = @pViewName;

    IF @vRootKey = 0 THEN 
         CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewNotFound,
            'CorrelateNodesAndObjects',
            'the business view not found ' || @pViewName
        );

    END IF;
   
    
    FOR BViewLevelLOOP AS BViewLevelCUR CURSOR FOR 
    select LEVEL AS LEVEL_R
                 from SAHARA.BVIEW_LEVEL
                where BVIEW_KEY = @vRootKey
                order by "LEVEL"
    DO

      CALL SP_IE_CorrelateNodesAtLevel(@pViewName, LEVEL_R, @pSessionKey);

    END FOR;

    CALL SP_IE_CorrelateObjects (@pViewName, @pSessionKey);
  END;
END;

grant execute on sahara_app.SP_IE_CorrelateNodesAndObjects to sahara_app;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_IE_LoadingBVStagingNode' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_IE_LoadingBVStagingNode;
END IF;


CREATE PROCEDURE SAHARA.SP_IE_LoadingBVStagingNode(IN @SQLString Long varchar)
BEGIN
declare @vSQL long varchar;
declare @vMessage long varchar;     
declare @FUNC varchar(64);

set @FUNC = 'SP_IE_LoadingBVStagingNode';

    set @vSQL = 'LOAD INTO TABLE SAHARA_APP.IE_NODE_
(
    "DISPLAY_NAME" ,
    "NODE_ID" ,
    "PARENT_ID" ,
    "DEPTH"
) USING VALUE ' || @SQLString  ||' WITH ROW LOGGING ;';
    
    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vSQL);

    execute immediate  @vSQL;

exception
  when others then
    set @vMessage = ERRORMSG( SQLCODE );
        --message ERRORMSG( SQLCODE ) to client;

    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vMessage);

END;
grant execute on sahara.SP_IE_LoadingBVStagingNode to sahara_app;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_IE_LoadingBVStagingTypes' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_IE_LoadingBVStagingTypes;
END IF;


CREATE PROCEDURE SAHARA.SP_IE_LoadingBVStagingTypes(IN @TableName varchar(64),IN @ColumnNames long varchar,IN @SQLString Long varchar)
BEGIN
declare @vSQL long varchar;
declare @vMessage long varchar;     
declare @FUNC varchar(64);

set @FUNC = 'SP_IE_LoadingBVStagingTypes';

    set @vSQL = 'LOAD INTO TABLE '|| @TableName || ' ( ' || @ColumnNames || ' ) USING  VALUE ' || @SQLString  ||' ESCAPES OFF WITH ROW LOGGING ;';
    
    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vSQL);

    execute immediate  @vSQL;

exception
  when others then
    set @vMessage = ERRORMSG( SQLCODE );
        --message ERRORMSG( SQLCODE ) to client;

    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vMessage);

END;
grant execute on sahara.SP_IE_LoadingBVStagingTypes to sahara_app;





/*  Lock sps  specific to Import Export module */


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_IE_AcquireExclusiveLock');

create procedure sahara.SP_IE_AcquireExclusiveLock(
     IN @SessionKey NUMERIC(10),
     IN @Timeout    integer default 0)
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SP_IE_AcquireExclusiveLock';
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call LockSession() for the CONFIGURATION module. If fails, it will raise
    -- an error, which will throw the exception which will return the
    -- control to the caller.

    CALL SP_Lock_LockSession('SAHARA_IE_CONTROL', @SessionKey);
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully acquired the lock');

    -- If we are here, then the lock was acquired, remember the session etc.
    SET @gIEHaveControl = 1;
    SET @gIEActiveSessionKey = @SessionKey;

END;

grant execute on sahara.SP_IE_AcquireExclusiveLock to sahara_app;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_IE_ReleaseExclusiveLock');
create procedure sahara.SP_IE_ReleaseExclusiveLock(
     IN @SessionKey NUMERIC(10))
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SP_IE_ReleaseExclusiveLock';
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Clear the state.

    SET @gIEHaveControl = 0;
    SET @gIEActiveSessionKey = null;

    -- Call UnlockSession() for the CONFIGURATION CONTROL module. 
    -- If this fails, it will raise an error, which will throw the 
    -- exception which will return the control to the caller.

    CALL SP_Lock_UnlockSession('SAHARA_IE_CONTROL', @SessionKey);

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully release the lock');
END;

grant execute on sahara.SP_IE_ReleaseExclusiveLock to sahara_app;

-------------------------------------------------------------------------------
CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','Fn_IE_GetActiveSessionKey');
CREATE FUNCTION sahara.Fn_IE_GetActiveSessionKey()
RETURNS NUMERIC(10)
NOT DETERMINISTIC
BEGIN
    RETURN @gIEActiveSessionKey;
END;

----------------------------------------------------------------------------

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_IE_RequireControl');
CREATE PROCEDURE sahara.SP_IE_RequireControl(
    IN @Func long varchar)
BEGIN
    IF ( @gIEHaveControl = 0) THEN
      BEGIN
        CALL SP_Error_RaiseApplicationError(
            @ENoIEControl,
            @Func || ': ' || 'caller does not have IE control lock'
        );
       END
    END IF;
END;




-- Function used for the export CLI
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_IE_CONCAT' and user_name(creator) = 'SAHARA_APP')
THEN
        DROP FUNCTION sahara_app.Fn_IE_CONCAT;
END IF;

CREATE FUNCTION sahara_app.Fn_IE_CONCAT(in @str1 long varchar , in @str2 long varchar)
RETURNS long varchar
BEGIN
    declare @string long varchar;
    set @string = (@str1 + @str2);
    RETURN @string;
END;




