use strict;
use warnings;

use ERPERLLIB;
require VRTS::VOMERPaths;

use Config::IniFiles;
use Cwd qw(realpath);
use DBI;
use ERLogging;
use File::Basename;
use File::Path;
use File::Spec;
use File::Temp qw(tempfile);
use Getopt::Std;
use NodeObject;
use SybaseHelpers;
use Tree::Nary;

# command line options
my %options = ();

# database handle
my $dbh = ();

# levels in the current business view
my @levels = ();

# level descriptions for the current business view
my @levelDescriptions = ();

# supported object types in the business view
my @types = ();

# suported types, organized by viewsetname
my %view_types = ();

# directory to the business view files
my $view_dir = ();

# import / export session key
my $session_key = 0;

#cleanup done
my $cleanupdone = 0;

# list of business view definitions
my @view_defs = ();

# list of rejected record files
my @reject_files = ();

# sql loader log files
my @sqlldr_log_files = ();

# business view assignment files
my @view_assignment_files = ();

# Options configuration file, as specified on the command line.
my $options_cfg;

my $ccer_home;
my $dev_null;

if ($^O eq 'MSWin32')
{
                require ERWinUtil;
                $ccer_home = VRTS::VOMERPaths::get_installpath(); 
                $dev_null = "NUL";
}
else
{
                $ccer_home = $ENV{CCER_HOME};
                $dev_null = "/dev/null";
}

# default options file in CCER_HOME/bin directory
my $default_opt_file = $ccer_home . '/bin/bv_import_csv.opt';

# if 1, writing to console is disable. All messages go to the
# log file only. undef, writing to console is enabled (default)
my $silent = undef;

# location to write log files.
my $log_dir = undef;

# the name of the log file
my $logfile = "import.log";

# logging level for this instance
my $log_level = 0;

# Oracle friendly name of the business view we're currently processing
my $view_oracle_name = ();

# Display name for the current business view
my $view_display_name = ();

# Description for the current business view
my $view_desc = ();

# business view node tree
my $bview_node_tree = ();

# number of objects found in the CSV file for the type
# key 1 - view display name
# key 2 - object type
# value - count of objects for that type
my %view_object_counts = ();

# number of objects successfully assigned to each view, organized by type
# key 1 - view display name
# key 2 - object type
# value - count of assigned objects for that type
my %view_assigned_counts = ();

# number of objects rejected because of invalid data for each view, organized by type
# key 1 - view display name
# key 2 - object type
# value - rejection count for that type
my %view_invalid_counts = ();

# number of objects rejected because the object didn't exist for each view, organized by type
# key 1 - view display name
# key 2 - object type
# value - rejection count for that type
my %view_unknown_counts = ();

# total row count - count of all rows in the CSV file (not including comment and blank lines )
my %view_row_counts = ();



#
# Hash of available columns based on type. Each object type has a set of columns that
# can uniquely identify an instance. In addition to the columns identified here, all
# types support an objectunique_key column which corresponds to the CC OID for the object.
#
# The keys of this hash represent all of the available assignment types currently
# supported by ER. The implication of this is that only objects of these types are
# supported by the ER database. Therefore, only these types of object assignments
# are supported. All other types will be ignored.
#
my %identity_columns_by_type =
(
   'HOST' => [ 'NAME', 'IP', 'OBJECT_UNIQUE_KEY' ],
   'ARRAY' => [ 'NAME', 'IP', 'OBJECT_UNIQUE_KEY' ],
   'SWITCH' => [ 'NAME', 'IP', 'OBJECT_UNIQUE_KEY' ],
   'APP_GROUP' => [ 'NAME', 'OBJECT_UNIQUE_KEY' ],
   'NETAPP_VOLUME' => [ 'FILER_NAME', 'PATH', 'OBJECT_UNIQUE_KEY' ],
   'NETAPP_SHARE' => [ 'FILER_NAME', 'PATH', 'OBJECT_UNIQUE_KEY' ],
   'NETAPP_QTREE' => [ 'FILER_NAME', 'PATH', 'OBJECT_UNIQUE_KEY' ],
   'NAS_FILESYSTEM' => [ 'FILER_NAME', 'NAS_FILESYSTEM_NAME', 'OBJECT_UNIQUE_KEY' ],
   'FILESYSTEM' => [ 'HOST_NAME', 'FILESYSTEM_NAME', 'OBJECT_UNIQUE_KEY' ],
   'VOLUME' => [ 'HOST_NAME', 'VOLUME_NAME', 'OBJECT_UNIQUE_KEY' ],
   'CLUSTER' => [ 'NAME', 'OBJECT_UNIQUE_KEY' ],
   'SERVICEGROUP' => [ 'CLUSTER_NAME', 'SERVICEGROUP_NAME', 'OBJECT_UNIQUE_KEY' ],
   'DISKGROUP' => [ 'HOST_NAME', 'DISKGROUP_NAME', 'OBJECT_UNIQUE_KEY' ],
);

my %map_nas_type_to_netapp_type =
(
   'NAS_VOLUME' => 'NETAPP_VOLUME',
   'NAS_SHARE' => 'NETAPP_SHARE',
   'NAS_QTREE' => 'NETAPP_QTREE'
);

#
# Helper array containing all types supported by ER.
#
my @supported_types = keys %identity_columns_by_type;

#
# Set the import result status for the session.
#
sub set_status {
   # do this on a separate connection so we get another transaction
   my $db = SybaseHelpers::connect();

   my( $viewsetname, $status ) = ( @_ );

   if( ! $viewsetname ) {
      $viewsetname = 'Unknown';
   }

   my $sql = "insert into SAHARA_APP.IE_STATUS(VIEWNAME, SESSION_KEY, STATUS_CODE) "
           . "values(?,?,?)";
   my $sth = $db->prepare($sql);
   $sth->execute( $viewsetname, $session_key, $status );
   $sth->finish;
   $db->commit();
   $db->disconnect();
}

#
# Add a log message to the sahara logging table
#
sub add_log_message {
   my( $viewsetname, $message ) = ( @_ );
   chomp( $message );

   # do this on a separate connection so we get another transaction
   my $db = SybaseHelpers::connect();
#   $db->do( "BEGIN call sahara.SP_Log_LogInfo($session_key,$viewsetname,$message); END;");


   my $sql = "begin execute sahara.SP_Log_Loginfo ?, ?, ? end";
#           .     "SAHARA.SP_Log_LogInfo"
#           .     "("
#           .         "pSessionKey => :parameter1, "
#           .         "pFunction => :parameter2, "
#           .         "pMessage => :parameter3"
#           .     "); "
#           . "end;";
   my $sth = $db->prepare($sql);
   $sth->bind_param(1, $session_key);
   $sth->bind_param(2, $viewsetname);
   $sth->bind_param(3, $message);
   $sth->finish();
   $db->disconnect();
}

#
# read in the business view definition file.  The definition file has the following
# sections and parameters:
#
# [Business View] -- overview of the view
#   Name          -- the business view name
#   Description   -- description for the view
#
# [Leveln]        -- defines the hierarchy level at depth n. One entry per level.
#   Name          -- level name
#   Description   -- level description
#
# [Object Types]  -- defines the object types supported by the view.
#   Types         -- comma separated list of supported types.
#                    (Host, Array, Switch, App_Group, NetApp_Share, NetApp_Qtree, NetApp_Volume)
#
# This proc will populate the following globals for the current view:
#     @levels
#     @levelDescriptions
#     @types
#     $view_display_name
#     $view_oracle_name
#     $view_description
#
# Returns 1 if the read was successful, 0 otherwise.
#
sub read_def_file {
   my $viewsetname = shift;
   my $def = File::Spec->catfile( $view_dir, $viewsetname . "." . "view" );

   ERLogging::write_log( "\nReading definition file \'" . $def . "\'\n", 1 );

   if( ! -f $def ) {
      ERLogging::write_log( "$def file does not exist.\n" );
      return 0;
   }

   my $cfg = new Config::IniFiles('-file' => "$def",
                                  '-nocase' => 1,
                                  '-allowcontinue' => 1);
   if( !$cfg ) {
      ERLogging::write_log( "Could not read definition file \'$def\'. $!\n" );
      return 0;
   }

   $view_display_name = $cfg->val( 'Business View', 'Name' );

   # Remove spaces from the business view name to avoid metamodel publish errors      
   $view_display_name=trim($view_display_name);

   if( !$view_display_name ) {
      ERLogging::write_log( "\nView name not defined in .view file\n" );
      return 0;
   }

   if( $view_display_name =~ /  /i ) {
                   ERLogging::write_log( "\nView name is not valid in .view file. View name cannot contain 2 or more consecutive spaces. Please remove consecutive spaces and try import.\n" );
       return 0;
   }

   $view_desc = $cfg->val( 'Business View', 'Description' );
   if( !$view_desc ) {
      ERLogging::write_log( "View description not defined in .view file.\n" );
                  # If the View description is not defined then no need to exit but
                  # still proceed with the assigments with having null in description column
   }

   $view_oracle_name  = SybaseHelpers::GetSybaseName( $view_display_name );

   my @levelSections  = grep { /^level/ } $cfg->Sections();

   for( my $i = 0; $i < @levelSections; ++$i ) {
      my $lvlname = $cfg->val( 'Level' . ($i + 1), 'Name' );

      # Remove spaces from the level name to avoid metamodel publish errors     
                  $lvlname=trim($lvlname);

                  if( $lvlname =~ /  /i ) {
                                 ERLogging::write_log( "\nLevel " . ( $i + 1 ) . " name is not valid in .view file. Level name cannot contain 2 or more consecutive spaces. Please remove consecutive spaces and try import.\n" );
                                 return 0;
                  }

      my $lvldesc = $cfg->val( 'Level' . ($i + 1), 'Description' );

      if( ! $lvlname ) {
         ERLogging::write_log( "Missing a level definition at level " . ( $i + 1 ) ."\n" );
         return 0;
      }
      else {
         push(@levels, $lvlname);
         push(@levelDescriptions, $lvldesc);
      }
   }

   my $line = $cfg->val( 'Object Types', 'Types' );
   if( !$line ) {
      ERLogging::write_log( "Object Types section not defined in the .view file.\n" );
      return 0;
   }
   @types = SybaseHelpers::parse_csv( $line );

   if( ! scalar( @types ) ) {
      ERLogging::write_log( "Object types not defined in the .view file.\n" );
      return 0;
   }

   # convert all types to upper case
   @types = map { uc } @types;

   foreach my $t ( @types ) {
      push( @{ $view_types{ $viewsetname } }, $t );
      ERLogging::write_log( "\nPushing OT name: $t on $viewsetname");
   }

   ERLogging::write_log( "Found business view: \'$view_display_name\'\n", 1 );
   ERLogging::write_log( "Levels: " . join(", ",@levels) . "\n", 1 );
   ERLogging::write_log( "Supported object types: " . join(", ", @types) . "\n", 1 );

   return 1;
}

#
# Populate the business view overview staging table. This staging tables contains
# the view name and description.
#
sub populate_view_overview_staging_table {
   my $viewsetname = shift;

   ERLogging::write_log( "Populating business view overview staging table...\n", 1 );

   my $sql = "insert into SAHARA_APP.IE_OVERVIEW_(BVIEW_NAME, DISPLAY_NAME, NOTE) "
           . "values(?,?,?)";
   my $sth = $dbh->prepare($sql);
   $sth->execute( $view_oracle_name, $view_display_name, $view_desc );
   add_log_message( $viewsetname, "Overview staging table loaded." );
}

#
# populate the types staging table. This table contains the types supported
# by this view.
#
sub populate_types_staging_table {
   my $viewsetname = shift;

   ERLogging::write_log( "Populating types staging table...\n", 1 );

   my $sql = "insert into SAHARA_APP.IE_TYPES_(TYPE_NAME) "
           . "values(?)";

   my $sth = $dbh->prepare($sql);
   foreach my $t ( @types ) {
                                if( $t =~ /NAS_VOLUME/i || $t =~ /NAS_SHARE/i || $t =~ /NAS_QTREE/i ) 
                                {
                                                $sth->execute( $map_nas_type_to_netapp_type{$t} );
                                }
                                else 
                                {
                                                $sth->execute( $t );
                                }
   }

   add_log_message( $viewsetname, "Types staging table loaded." );
}

#
# Create and populate the view level staging table. The view level staging table
# contains one row for each level in the view. This corresponds to the BVIEW_LEVEL
# production table in the sahara database.
#
sub populate_level_staging_table {
   my $viewsetname = shift;

   ERLogging::write_log( "Populating level staging table...\n", 1 );

   my $sql = "insert into SAHARA_APP.IE_LEVELS_(DEPTH, LEVEL_NAME, LEVEL_DISPLAY_NAME, NOTE) "
           . "values(?,?,?,?)";
   my $sth = $dbh->prepare($sql);

   my @prevColumnNames = ();
   for( my $i=0; $i < scalar( @levels ); ++$i ) {

      my $display_name = $levels[ $i ];
      my $description = $levelDescriptions[ $i ];
      my $level_name = SybaseHelpers::GetSybaseName( $display_name, 30, \@prevColumnNames );
      $sth->execute( $i + 1, $level_name, $display_name, $description );
   }

   add_log_message( $viewsetname, "Business view level staging table loaded.", 1 );
}
sub Load_Into_Staging_Table {
                my $TableName = shift;
                my $csv_file_name = shift;
                my $column_names = shift;

                my $sql = "LOAD TABLE $TableName from $csv_file_name"; 
                my $sth = $dbh->prepare($sql);
                $sth->execute();
}

#
# Populate the view assignment staging table on the underlying CSV file.
#
sub populate_view_assignment_staging_tables {
   my $viewsetname = shift;
  # my $ctlstr;

   ERLogging::write_log( "Populating business view assignment staging tables...\n", 1 );

   # create the parameters file for SQL*Loader (automatically deleted later)

   # get a list of all view_ files
   opendir( DEFS, $view_dir ) or die "Cannot open directory for reading. $!";
   my @view_files = grep { /^${viewsetname}_.*\.csv/i } readdir( DEFS );
   closedir( DEFS );

   
foreach my $type ( @types )
   {
      # find the support type
      my $found = 0;
      foreach my $view_data_file ( @view_files ) {
         
         if( grep( { /${viewsetname}_${type}.csv/i } $view_data_file ) ) {
            $found = 1;

			 my $fn = $viewsetname . "_" . $type;
             my $fullpath = File::Spec->catfile( $view_dir, $view_data_file );
             my $badfile = File::Spec->catfile( $log_dir, "${fn}_sqlldr.bad" );
             my $disfile = File::Spec->catfile( $log_dir, "${fn}_sqlldr.dsc" );
             my $logfile = File::Spec->catfile( $log_dir, "${fn}_sqlldr.log" );
             my $rejfile = File::Spec->catfile( $log_dir, "${fn}.rej" );
             my $csvfile = "${fullpath}.csv";
			 my $orgtype=$type;
             my $ctlstr;

			 if( $type =~ /NAS_VOLUME/i || $type =~ /NAS_SHARE/i || $type =~ /NAS_QTREE/i ) 
			 {
							$type=$map_nas_type_to_netapp_type{$type};
			 }

			 my $tablename = "SAHARA_APP.IE_${type}_" ;
             push( @view_assignment_files, $view_data_file );

             my $column_names = "   \"NODE_ID\" ,";
             my @prevColumnNames = ();

             my @identity_columns = @{$identity_columns_by_type{$type}};
             foreach my $column (@identity_columns)
             {
                $column_names .= "\n   \"" . SybaseHelpers::GetSybaseName($column, 30, \@prevColumnNames) . "\",";
             }
             chop($column_names); # remove trailing comma



             NodeObject->traverse($bview_node_tree,
                                  $Tree::Nary::PRE_ORDER,
                                  $Tree::Nary::TRAVERSE_LEAFS,
                                  -1,
                                  sub
                                  {
                                     my ($node) = @_;
                                        $type=$orgtype;

                                     if($node->{data}->{type} ne $type)
                                     {
                                         return 0;
                                     }

                                     my $data = $node->{data}->{name};
                                     my $node_id = $node->{data}->{id};

                                     $ctlstr .= $node_id . "," . $data . "\n";
                                     return 0;
                                  },
                                  undef);
         $ctlstr=~ s/'/''/g;

                my $datavalue = "\'$ctlstr\'";

        my $sql1 = "begin execute sahara.SP_IE_LoadingBVStagingTypes ?, ?, ? end";
   my $sth = $dbh->prepare($sql1);
   $sth->bind_param( 1, $tablename );
   $sth->bind_param( 2, $column_names );
   $sth->bind_param( 3, $datavalue );
   $sth->execute();
   $sth->finish;

     

   #print " The tablename: $tablename, csvfilename : $csvfile Column name : $column_names\n";
         }
      }

      if( $found == 0 ) {
         ERLogging::write_log("\nThis business view supports $type objects but a ${viewsetname}_${type}.csv\n" .
                   "file was not found. No object assignments for $type objects are defined.\n\n" );

         add_log_message( $viewsetname, "View supports $type but ${viewsetname}_${type}.csv not found." );
      }
   }

   add_log_message( $viewsetname, "Business view assignment tables loaded." );
}

#
# Print out an individual node in the tree in human readable format
#
sub print_node {
   my ( $node ) = ( shift );
   my $depth = NodeObject->depth( $node ) - 1;

   if( $depth <= 0 ) {
      return;
   }

   if( $depth > 1 ) {
     print " " x ($depth * 3);
   }

   if( $node->{data}->{type} eq 'Node' ) {
      print $node->{data}->{name};
      print "," . $node->{data}->{id} . ",";

      if( $node->{parent}->{data} ) {
         print $node->{parent}->{data}->{id};
      }
   }
   else {
      print "( " . $node->{data}->{name} . " )";
   }

   print "\n";

   return 0;
}

#
# Write out the entire tree
#
sub dump_tree {
   my ( $root ) = ( shift );

   if( $root ) {
      NodeObject->traverse( $$root,
                            $Tree::Nary::PRE_ORDER,
                            $Tree::Nary::TRAVERSE_ALL,
                            -1,
                            \&print_node,
                            undef);
   }
   print "\n";
}

sub writeNodeToControlString {
   my ( $node ) = ( shift );
   my ( $ctlstr ) = ( shift );

   my $root = $node->get_root( $node );

   bless $root,"NodeObject";

   # note - Tree:Nary has a default root. We don't want to include this in the depth calculations.
   my $depth = $root->depth( $node ) - 1;

   # don't write out the root or non-'Node' entries
   if( $node->{data}->{id} != 0 && $node->{data}->{type} eq 'Node' ) {

      if( $node->{data}->{name} =~ /^Unspecified/ ) {
         $$ctlstr .= 'N/A,';
      }
      else {
         $$ctlstr .= "\'\'" . $node->{data}->{name} . "\'\',";
      }

      $$ctlstr .= $node->{data}->{id} . ",";

      if( $node->{parent} && $node->{parent}->{data}->{id} != 0 ) {
         $$ctlstr .= $node->{parent}->{data}->{id};
      }
      $$ctlstr .= "," . $depth;
      $$ctlstr .= "\n";
   }
   return 0;
}

#
# Populate the node staging table on the underlying CSV files.
#
sub populate_node_staging_table
{
    my ($viewsetname) = @_;
    my $datastr; 


    ERLogging::write_log("Populating business view node staging table...\n", 1);


    # get a list of all view_ files
    opendir(DEFS, $view_dir) or
        die("Failed opening view directory $view_dir for reading: $!\n");
    my @view_files = grep { /^${viewsetname}_.*\.csv/i } readdir(DEFS);
    closedir(DEFS);

    my $fn = $viewsetname . "_node";
    my $badfile = File::Spec->catfile($log_dir, "${fn}_sqlldr.bad");
    my $disfile = File::Spec->catfile($log_dir, "${fn}_sqlldr.dsc");
    my $logfile = File::Spec->catfile($log_dir, "${fn}_sqlldr.log");
             my $csvfile = "${fn}.csv";
                     my $tablename = "SAHARA_APP.IE_NODE_" ;

    my $ctlstr .= qq(
LOAD INTO TABLE SAHARA_APP.IE_NODE_ 
(
    "DISPLAY_NAME" ,
    "NODE_ID" ,
    "PARENT_ID" ,
    "DEPTH" 
) USING VALUE );


    my @unique_branches = ();
    my %assignments = ();

    foreach my $type (@types)
    {
        # find the support type
        my $non_comment_count = 0;
        foreach my $view_data_file (@view_files)
        {
            if(!grep({/${viewsetname}_${type}.csv/i} $view_data_file))
            {
                next;
            }
            # else

            my $fullpath = File::Spec->catfile($view_dir, $view_data_file);

                                                my @identity_columns = ();
                
                                                if( $type =~ /NAS_VOLUME/i || $type =~ /NAS_SHARE/i || $type =~ /NAS_QTREE/i ) 
                                                {
                                                                @identity_columns = @{$identity_columns_by_type{$map_nas_type_to_netapp_type{$type}}};
                                                }
                                                else
                                                {
                                                                @identity_columns = @{$identity_columns_by_type{$type}};
                                                }                                                              

            ERLogging::write_log( "Loading \'$fullpath\'...\n", 1 );

            my $expectedColumns = scalar(@levels) + scalar(@identity_columns);

            # load the data file, remove any blank lines and comments as we go.  Also make
            # sure there are enough columns defined in the file
            open(CSV, "<$fullpath" ) or
                die("Failed opening data file '$fullpath': $!\n");

            my $lineNumber = 0;
            while(my $line = <CSV>)
            {
                $lineNumber++;

                # Comment lines start with '#' or ';'. Also weed out
                # empty lines.
                if(!defined($line) || grep {/^#|^;|^\n|^\r/} $line)
                {
                    next;
                }
                # else

                # we have a non-comment line
                $non_comment_count++;
                $line =~ s/\'/\\'/g;
                #if ($line =~ (m/\w:\\/) | (m/\@\w:\\/) ) {
                                                                #             $line =~ s/:\\/:\\\\/g;
                                                                #}
                                                                if ($line =~ (/\W\\/) ) {
                                                                                $line =~ s/\\/\\\\/g;
                                                    }

                                                                # make sure the line has the required number of columns
                my @cols = SybaseHelpers::parse_csv($line);
                if(scalar(@cols) != $expectedColumns)
                {
                    my $msg = "line $lineNumber, too "
                            . ((scalar(@cols) < $expectedColumns) ? "few" : "many")
                            . " columns: expected $expectedColumns, found " . scalar(@cols) . ".\n";
                    chomp($line);

                    my $rejfile = File::Spec->catfile($log_dir, "${viewsetname}_${type}.rej" );
                    open( REJECT, ">>$rejfile" ) or
                        die("Failed opening rejected record file, $rejfile: $!\n");
                    print(REJECT "$line\n");
                    close(REJECT);

                    $view_invalid_counts{$viewsetname}{$type}++;

                    ERLogging::write_log($msg, 1);
                    ERLogging::write_log("$line\n", 1);
                    add_log_message($viewsetname, "${viewsetname}_${type} : " . $msg);

                    push(@reject_files, $rejfile)
                       unless grep({/\Q$rejfile/} @reject_files);

                    next;
                }
                # else

                my $branch = "";

                # levels below root start at 1
                for(my $i = 1; $i <= scalar(@levels); ++$i)
                {
                    my $node_name = $cols[$i-1];
                    if($node_name eq "")
                    {
                        $node_name = 'Unspecified ' . $levels[$i-1];
                    }

                    $branch .= "\'$node_name\',";
                }

                # remove trailing comma
                chop($branch);
                push(@unique_branches, $branch) unless grep {/\Q$branch/} @unique_branches;

                # object for the branch
                my $object = undef;
                for(my $i = scalar(@levels); $i < $expectedColumns; ++$i)
                {
                    $object .= "\'" . $cols[$i] . "\',";
                }
                chop( $object );
                                #print "Displaying object : $object \n";

                # check for null object
                my $pattern = "\'\'," x ( scalar( @identity_columns ) - 1 ) . "\'\'";
                if( ! ( $object eq $pattern ) )
                {
                    my $ref;
                    $ref->{name} = $object;

                    # object don't get assigned ids here.
                    $ref->{id} = -1;
                    $ref->{type} = $type;

                    push( @{ $assignments{ $branch } }, $ref );

                    # track the total number of objects in this view, by type
                    $view_object_counts{ $viewsetname }{ $type }++;
                }
            }

            close(CSV);

        } # foreach $data_file

        $view_row_counts{ $viewsetname }{$type} = $non_comment_count;

    } # foreach $type

    # create a tree out of the unique branch list
    my $nextId = 0;
    my $nd;
    $nd->{name} = '**ROOT**';
    $nd->{id} = $nextId++;
    $nd->{type} = 'Node';

    $bview_node_tree = new NodeObject( $nd );

    foreach my $branch ( @unique_branches )
    {
        my @cols = SybaseHelpers::parse_csv($branch);

        my @assigned = ();
        if( $assignments{ $branch } )
        {
            @assigned = @{ $assignments{ $branch } };
        }

        my $parent = $bview_node_tree;
        my $i = 1;
        foreach my $col ( @cols )
        {
            my $ref;
            $ref->{name} = $col;
            $ref->{id} = $nextId++;
            $ref->{type} = 'Node';
            $ref->{depth} = $i;

            my $tmp = NodeObject->find($parent,
                                       $Tree::Nary::IN_ORDER,
                                       $Tree::Nary::TRAVERSE_ALL,
                                       $ref);

             # if a node with the given name doesn't exist, or if it does exist
             # and is at a different depth, inject a new node.
             if(! $tmp || $tmp->{data}->{depth} != $ref->{depth})
             {
                 $parent = $bview_node_tree->insert_data( $parent, -1, $ref );
             }
             else
             {
                 # otherwise skip this node and move on to the next.
                 $parent = $tmp;
             }
             $i++;
        }

        # don't attach objects to empty nodes. Find the next node up the branch that
        # has a name assigned to it.
        while( $parent && ( ! $parent->{data}->{name} || $parent->{data}->{name} =~ m/^Unspecified/ ) )
        {
            $parent = $parent->{parent};
        }

        # nothing to assign to.
        if( ! $parent )
        {
            next;
        }

        if( @assigned )
        {
            foreach my $obj ( @assigned )
            {
                if( $obj )
                {
                                  #print "Assigning id : $obj \n";
                   # set the id to that of the parent of the object
                   $obj->{id} = $parent->{data}->{id};
                   $bview_node_tree->insert_data( $parent, -1, $obj );
                }
            }
        }
    }

    # trim off any Unspecified leaf nodes. They're not needed.
    my $normalized = 0;
    while( ! $normalized )
    {
        $normalized = 1;
        NodeObject->traverse($bview_node_tree,
                             $Tree::Nary::PRE_ORDER,
                             $Tree::Nary::TRAVERSE_LEAFS,
                             -1,
                             sub
                             {
                                my ($node) = @_;

                                if($node->{data}->{type} ne 'Node')
                                {
                                    return 0;
                                }

                                if($node && $node->{data}->{name} =~ m/^Unspecified/)
                                {
                                    my $parent = $node->{parent};

                                    if($parent)
                                    {
                                        $parent->unlink($node);
                                    }
                                    $normalized = 0;
                                }
                                return 0;
                            },
                            undef);
   }

   # DEBUG: dump_tree(\$bview_node_tree);

    # add the nodes to the control string
  NodeObject->traverse($bview_node_tree,
                       $Tree::Nary::PRE_ORDER,
                       $Tree::Nary::TRAVERSE_ALL,
                       -1,
                       \&writeNodeToControlString,
                       \$datastr);
            #ERLogging::write_log( "Loading \'$fullpath\'...\n", 1 );
#   print " The tablename: $tablename, csvfilename : $csvfile Control string :$ctlstr\n";
  

#             my $sql = $ctlstr."\'$datastr\'";
                my $sql = "\'$datastr\'";
#             print "$sql\n";
#     my $dbh = SybaseHelpers::connect();

   my $sql1 = "begin execute sahara.SP_IE_LoadingBVStagingNode ? end";
   my $sth = $dbh->prepare($sql1);
   $sth->bind_param( 1, $sql );
   $sth->execute();
   $sth->finish;

  # $dbh->do("begin CALL SAHARA.SP_IE_LoadingBVStaging($sql); end;"); 
#   $dbh->disconnect();

    #Load_Into_Staging_Table($tablename,$csvfile,"Columname");

   add_log_message($viewsetname, "Business view node staging table loaded.");
}

#
# initialize for the next run by cleaning up tables
# and acquiring the session lock.
#
sub init_db {

 my @results = ();

   ERLogging::write_log( "Initializing a database connection...\n", 1 );

   # disable auto commit. Make sure we raise errors so we can rollback
   $dbh = SybaseHelpers::connect();

   $dbh->do("BEGIN call sahara.SP_saharaAPIInit(); END;") || die "Error while initializing APIs : $dbh->errstr";


   # acquire a session key
   ERLogging::write_log( "Acquiring session key...\n", 1 );
   my $sql = "begin select SAHARA.fn_Session_BeginIESession() end;";
   my $hdl = $dbh->prepare($sql);
   $hdl->execute();
   while (@results = $hdl->fetchrow()) {
        $session_key = $results[0];
   }
   $hdl->finish;


   ERLogging::write_log( "Sahara database session key: $session_key\n", 1 );

   # acquire the session lock
   ERLogging::write_log( "Acquiring Import/Export exclusive lock...\n", 1 );
                # Change this to the IE exclusive lock once its ported
   $dbh->do( "begin CALL sahara.SP_IE_AcquireExclusiveLock($session_key); end;");

   # acquire the business view session lock
   ERLogging::write_log( "Acquiring Configuration exclusive lock...\n", 1 );
   $dbh->do( "begin CALL sahara.SP_Config_AcquireExclusiveLock($session_key);  end;");
}

#
# Clear out our temporary staging tables
#
sub cleanup_staging_tables {

   ERLogging::write_log( "Cleaning staging tables...\n", 1 );

   # disable raise error for now
   $dbh->{RaiseError} = 0;
   $dbh->{PrintError} = 0;

   my $sql = "truncate table SAHARA_APP.IE_OVERVIEW_";
   my $sth = $dbh->prepare($sql);
   $sth->execute();

   $sql = "truncate table SAHARA_APP.IE_LEVELS_";
   $sth = $dbh->prepare($sql);
   $sth->execute();

   foreach my $type ( @supported_types ) {
      $sql = "truncate table SAHARA_APP.IE_${type}_";
      $sth = $dbh->prepare($sql);
      $sth->execute();
   }

   $sql = "truncate table SAHARA_APP.IE_TYPES_";
   $sth = $dbh->prepare($sql);
   $sth->execute();

   $sql = "truncate table SAHARA_APP.IE_NODE_";
   $sth = $dbh->prepare($sql);
   $sth->execute();

   $dbh->{RaiseError} = 1;
   $dbh->commit();
}

#
# clean up the database connections
#
sub cleanup_db {
   # clear the Configuration control lock
        if( $cleanupdone ne 1 )
        {
                print "cleanupdone value : $cleanupdone\n";
   my $sql = "begin CALL sahara.SP_Config_ReleaseExclusiveLock(); end;";
   my $hdl = $dbh->prepare($sql);
   $hdl->execute();
   $dbh->commit();

   # clear the I/E lock
   $sql = "begin CALL SAHARA.SP_IE_ReleaseExclusiveLock($session_key); end;";
   $hdl = $dbh->prepare($sql);
   $hdl->execute();
   $dbh->commit();

   # close the session
                # Change this to the IE exclusive lock once its ported
   $dbh->do( "begin CALL sahara.fn_Session_EndSession($session_key); end;");
                                $cleanupdone = 1;
                                }
                }

                #
                # Correlate the staged view from the CSV file against the existing views
                #
                sub correlate_view
                {
                    my $overwrite = 1;
                    ERLogging::write_log( "Correlating business view...\n", 1 );

                    if(defined($options_cfg))
                    {
                                my $yn = $options_cfg->val('Import Options', 'Overwrite_Existing_Views');

                                if(($yn =~ /^y/i) || ($yn =~ /^1/i))
                                {
                                    $overwrite = 1;
                                }
                    }

                    ERLogging::write_log( "Correlating business view...\n", 1 );

                    my $sql;
                    my $sth;

                    eval
                    {
                    $sql = "begin execute SAHARA_APP.SP_IE_CorrelateImportedViews ?  end";
                    #$sql = "begin CALL SAHARA_APP.SP_IE_CorrelateImportedViews()  ; end;";
                    $sth = $dbh->prepare($sql);
                    $sth->bind_param(1, $overwrite);
                    $sth->execute();
                    $sth->finish;
                   };

                    eval
                    {
                                $sql = "begin CALL SAHARA.SP_BVIEW_PublishBV(); end;";
                                $sth = $dbh->prepare($sql);
                                $sth->execute();
                    };

    if($@)
                    {
                                ERLogging::write_log("This business view failed to publish.  Verify " .
                                                                     "that the view definition is valid.  " .
                                                                     "Skipping view '$view_display_name'...\n");
                                return 0;
                    }

                    return 1;
                }

                #
                # Correlate business view nodes
                #
                sub correlate_nodes_and_objects {

                   ERLogging::write_log( "Correlating business view nodes...\n", 1 );

                #   $dbh->do( "begin CALL SAHARA_APP.SP_IE_CorrelateNodesAndObjects($view_oracle_name,$session_key);  end;") || die " DBI error $dbh->errstr";
                   my $sql = "begin execute SAHARA_APP.SP_IE_CorrelateNodesAndObjects ?, ? end";
                   my $hdl = $dbh->prepare($sql);
                    $hdl->bind_param(1, $view_oracle_name);
                    $hdl->bind_param(2, $session_key);
                    $hdl->execute();
                    $hdl->finish;
                }

                #
                # Write out a readable import summary report.
                #
                sub write_summary_report {
                   my $viewsetname = shift;
                   my $option_file = shift || 'none';

                   ERLogging::write_log( "\n\nView import status information\n" );
                   ERLogging::write_log( "-" x 79 );
                   ERLogging::write_log( "\nView directory    : " . $view_dir . "\n" );
                   ERLogging::write_log( "Options file      : " . $option_file . "\n" );
                   ERLogging::write_log( "View definition   : ${viewsetname}.view\n" );
                   ERLogging::write_log( "Assignment files  : " . join( "\n                    ", @view_assignment_files ) . "\n" );
                   ERLogging::write_log( "Rejected records  : " . join( "\n                    ", @reject_files ) . "\n" );
                   ERLogging::write_log( "Log file          : " . File::Spec->catfile( $log_dir, $logfile ) . "\n" );
                   ERLogging::write_log( "\n\n" );

                   # Display log messages for this view
                   my $sql = "select \'TIMEi\', FUNCTION_NAME, \'MESSAGE\' "
                                   .   "from SAHARA.MESSAGE_LOG "
                                   .  "where SESSION_KEY = ? and FUNCTION_NAME= ? "
                                   .  "order by \'TIME\', SESSION_KEY";
                   my $sth = $dbh->prepare($sql);
                   $sth->execute( $session_key, $viewsetname );

                   ERLogging::write_log( "\n\n --- Business view import log messages\n\n" );
                   ERLogging::write_log( "View Set" );
                   ERLogging::write_log( " " x 12 );
                   ERLogging::write_log( "Log Message\n" );
                   ERLogging::write_log( "-" x 79 );
                   ERLogging::write_log( "\n" );
                   my @data = ();
                   while ( @data = $sth->fetchrow_array() ) {
                      ERLogging::write_log( "$data[1]" );
                      ERLogging::write_log( " " x ( 20 - length( $data[1] ) ) );
                      ERLogging::write_log( "$data[2]\n" );
                   }

                   # display view import status information
                   $sql  = "select VIEWNAME, STATUS_CODE "
                                 .   "from SAHARA_APP.IE_STATUS "
                                 .  "where SESSION_KEY = ?";
                   $sth = $dbh->prepare($sql);
                   $sth->execute( $session_key );

                   ERLogging::write_log( "\n --- Business view import status\n\n" );
                   ERLogging::write_log( "View Set" );
                   ERLogging::write_log( " " x 14 );
                   ERLogging::write_log( "Status Message\n" );
                   ERLogging::write_log( "-" x 79 );
                   ERLogging::write_log( "\n" );
                   while( @data = $sth->fetchrow_array() ) {
                      ERLogging::write_log( "$data[0]" );
                      ERLogging::write_log( " " x ( 22 - length( $data[0] ) ) );
                      ERLogging::write_log( "$data[1]\n" );
                   }
                }

                #
                # Write any rejected records to the view.rejected file
                #
                sub save_rejected_records {
                   my $viewsetname = shift;

                   foreach my $type ( @types ) {
                      my $once = 0;
                      my $fn = $viewsetname . "_" . $type;
                                  my @columns= ();
                                  my $sql;
                      my $rejfile = File::Spec->catfile( $log_dir, "${fn}.rej" );
                      open REJECTED, ">>${rejfile}" or die "Couldn't open $rejfile for writing. $!\n";

                                  if( $type =~ /NAS_VOLUME/i || $type =~ /NAS_SHARE/i || $type =~ /NAS_QTREE/i ) 
                                  {
                                                                @columns = @{$identity_columns_by_type{$map_nas_type_to_netapp_type{$type}}};
                                                                $sql = "select BVIEW_NODE_KEY, " . join(", ", @columns ) . " "
                                      .   "from SAHARA_APP.IE_${map_nas_type_to_netapp_type{$type}}_ "
                                      .  "where REJECTED = 'T'";
                                  }
                                  else
                                   {
                                                                @columns = @{$identity_columns_by_type{$type}};
                                                                $sql = "select BVIEW_NODE_KEY, " . join(", ", @columns ) . " "
                                      .   "from SAHARA_APP.IE_${type}_ "
                                      .  "where REJECTED = 'T'";
                                   }
      
                      my $sth = $dbh->prepare($sql);
                      $sth->execute();

                      my @data = ();
                      while ( @data = $sth->fetchrow_array() ) {

                                                 $view_unknown_counts{ $viewsetname }{ $type }++;

                                 if( !$data[0] ) {
            # not much we can do without the node_key, move on to the next row
            next;
         }

         if( !$data[1] ) {
            $data[1] = "";
         }
         if( !$data[2] ) {
            $data[2] = "";
         }
         if( !$data[3] ) {
            $data[3] = "";
         }

         # get the branch from the BVS table
         my @branch = ();
         my $sql2 = "select ";
         foreach my $level ( @levels )
         {
            my $level_oracle_name = SybaseHelpers::GetSybaseName($level, 30);

            $sql2 .= "\"" . $level_oracle_name . "\",";
         }

         # remove trailing comma
         chop($sql2);
         $sql2 .= " from SAHARA.BVS_$view_oracle_name "
               .  "where BVIEW_NODE_KEY=?";
         my $sth2 = $dbh->prepare_cached($sql2);
         $sth2->execute( $data[0] );

         my $line = "";
         while( @branch = $sth2->fetchrow_array() ) {

            for( my $i = 0; $i < scalar( @branch ); ++$i ) {
               my $tmp = "";
               if( !$branch[$i] || $branch[$i] eq 'N/A' ) {
                  $tmp = "";
               }
               else {
                  $tmp = $branch[$i];
               }
               $line .= "$tmp,";
            }
         }

         if( $data[0] ne '' || $data[1] ne '' || $data[2] ne '' ) {
            my $msg = "$type not in ER database: $data[1],$data[2],$data[3]\n";

            if( $type eq 'APP_GROUP' ) {
               # app group uses name and OID only
               $line .= "$data[1],$data[3]\n";
            }
            elsif( $type eq 'HOST' || $type eq 'ARRAY' || $type eq 'SWITCH' ) {
               # name, ip, OID
               # quote IP addresses
               $line .= "$data[1],\"$data[2]\",$data[3]\n";
            }
            elsif( $type eq 'NAS_VOLUME' || $type eq 'NAS_SHARE' || $type eq 'NAS_QTREE' ) {
               # filer name, path, oid
               $line .= "$data[1],\"$data[2]\",$data[3]\n";
            }
            elsif( $type eq 'CLUSTER' ) {
               # name, oid
               $line .= "$data[1],$data[3]\n";
            }
            elsif( $type eq 'SERVICEGROUP' ) {
               # cluster_name, servicegroup_name, oid
               $line .= "$data[1],$data[2],$data[3]\n";
            }

            print REJECTED $line;

            if( !$once ) {
               $once = 1;
               my $tmp = "Some objects defined in the CSV file were not found "
                       . "in the ER database and were rejected. "
                       . "See $rejfile for a list of these objects\n";
               ERLogging::write_log( $tmp, 1 );
               add_log_message( $viewsetname, $tmp );
            }

            push(@reject_files, $rejfile)
               unless grep( { /\Q${rejfile}/ } @reject_files );
         }
      }
      close REJECTED;
   }
}

#
# Get a count of all the records for each supported type - assigned and unknown.
#
sub get_record_counts {
   my $viewsetname = shift;
   my $orgtype;

   foreach my $type ( @types ) {
      $orgtype=$type;
                  if( $type =~ /NAS_VOLUME/i || $type =~ /NAS_SHARE/i || $type =~ /NAS_QTREE/i ) 
                  {
                                $type=$map_nas_type_to_netapp_type{$type};
                  }

      # assigned records
      my $sql = "select COUNT(*) "
              .   "from SAHARA_APP.IE_${type}_ "
              .  "where REJECTED='F'";
      my $sth = $dbh->prepare($sql);

      my $assigned = 0;
      $sth->execute();
      $sth->bind_col( 1,\ $assigned );
      #$sth->execute();
      $sth->fetch();

      $type=$orgtype;        
                  $view_assigned_counts{ $viewsetname }{ $type } = $assigned;

                  if( $type =~ /NAS_VOLUME/i || $type =~ /NAS_SHARE/i || $type =~ /NAS_QTREE/i ) 
                  {
                                $type=$map_nas_type_to_netapp_type{$type};
                  }

      # unknown records
      $sql = "select COUNT(*) "
           .   "from SAHARA_APP.IE_${type}_ "
           .  "where REJECTED='T'";
      $sth = $dbh->prepare($sql);
      my $unknown = 0;
      $sth->execute();
      $sth->bind_col( 1, \$unknown );
      $sth->fetch();

      $type=$orgtype;
                  $view_unknown_counts{ $viewsetname }{ $type } = $unknown;
   }
}

#
# Remove spaces from the names
#
sub trim($)
{
                my $string = shift;
                $string =~ s/^\s+//; # remove leading spaces
                $string =~ s/\s+$//; # remove trailing spaces
                return $string;
}

###############################################################################
#
# main entry point
#
my $usage =
q(Usage: bv_import_csv.pl -d directory [-v .view file] [-x display name] [-o option file] [-u] [-a] [-?] [-h]

Flags:
   -d  directory     : This required flag lets you define the directory
                       containing the business view CSV files you want
                       to import.
   -v  .view file    : This optional flag lets you define the filename
                       of the business view '.view' file to import.  Only
                       the business view set defined by this '.view' file
                       will be imported.
   -o  options_file  : This optional flag lets you define the name of
                       an import options file.  The import options file
                       is used to provide optional parameters to the
                       import process.
   -u                : This optional flag, if set, tells the script to
                       execute the update_metamodel.pl script to publish
                       the imported business views to the cognos metamodel.
   -a                : This optional flag tells the script to perform the
                       object assignment phase only.  Use this flag if you
                       want to assign additional objects to an existing
                       ER view.  No changes to the business view structure
                       nor the metamodel are performed when this flag is
                       set.  The script will fail of If the view defined in
                       the .view file does not exist in the ER database.
                       If both -a and -u are specified, the -u flag is ignored.
   -x                : Instead of importing a new view, this flag indicates the
                       specified view should be deleted.
   -?                  Print this usage message.
   -h                  Print this usage message.

The bv_import_csv.pl script is used to import Enterprise Reporter business
views defined in a comma separated value (CSV) file format.  An example
import set and options file can be found in the VRTSccer/examples directory.

This script looks for '.view' files in the directory specified by the -d command
line option.  Each '.view' file defines the structure of a single Enterprise
Reporter business view.  If the -d option is not specified, the script will exit.

This script can import all of the business views found in the working directory
or a single business view by specifying the business view '.view' filename
using the -v option.
);

getopts("ad:hto:uv:x:?", \%options);

if($options{'?'} || $options{'h'})
{
    print($usage);
    exit(0);
}



my $view_def_file = $options{'v'};
my $option_file = $options{'o'};
my $run_update_script = $options{'u'};
my $assignment_only = $options{'a'};
my $delete_view = $options{'x'};

# the directory option is required
# Deleting views don't require directory name
$view_dir = $options{'d'};
if(!defined($delete_view) && !defined($view_dir))
{
    print(STDERR "The business view directory option (-d) is required.\n\n$usage");
    exit(-1);
}


if(defined($option_file))
{
    $options_cfg = new Config::IniFiles('-file' => $option_file,
                                        '-nocase' => 1,
                                        '-allowcontinue' => 1)
       or die("Failed opening options file $option_file: $!\n");

    $silent = $options_cfg->val('import options', 'silent');
    $log_dir = $options_cfg->val('import options', 'log_directory');
    $log_level = $options_cfg->val('import options', 'log_level');
}

if($options{'t'})
{
   $silent = 1;
}

if( ! $log_dir ) {

   #$log_dir = $ENV{'ER_IMPORT_LOG_DIR'};
   $log_dir = VRTS::VOMERPaths::get_apppath();
   $log_dir = $log_dir  . "\\logs\\business_view";

   if( ! $log_dir ) {
      $log_dir = "log";
   }
}


my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
my $start_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d",
                          $year+1900,$mon+1,$mday,$hour,$min,$sec );

ERLogging::init( $log_dir, 'import', $silent, $log_level);

$log_dir = $ERLogging::log_dir;
$logfile = $ERLogging::logfile;


if( $delete_view ) {
   ERLogging::write_log( "-" x 79 );
   ERLogging::write_log( "\nStarting new import session at " . $start_time . "\n" );
   ERLogging::write_log( "Attempting to delete view $delete_view\n" );
   ERLogging::write_log( "-" x 79 );
   
   eval {
      

     # First delete the view from metamodel. 
     # Update metamodel script reads view names from database
     # make sure to delete the view from the metamodel 
     ERLogging::write_log("\nExecuting update_metamodel.pl script\n");
     my $iscript = "";
     $^O eq 'MSWin32' ? ($iscript .= "perl.exe ") : ($iscript .= "perl ");
    
     $iscript .= "update_metamodel.pl -l $log_dir -x \"$delete_view\"";
    
     system($iscript);
     if($? == -1 || $? & 127)
     {
       ERLogging::write_log( "Error executing $iscript: $!\n");
       ERLogging::write_log( "Exiting import process\n");
       exit -1;
     }            
      ERLogging::write_log( "Deleting business view $delete_view from Cognos metamodel\n");
        
      ERLogging::write_log( "Deleting business view $delete_view from database\n");
              
        
      # initialize the database and acquire configuration lock  
      init_db();
      
       # get the view key given the display name
      my $vk = 0;      
      
      my $sql = "select BVIEW_KEY FROM SAHARA.BVIEW where DISPLAY_NAME = ?";
      my $sth = $dbh->prepare($sql);
      $sth->bind_param( 1, $delete_view );
      $sth->bind_col( 1, \$vk );
      $sth->execute();
      $sth->fetch();
      
      if( $vk == 0 ) {
          # if database does not contain the business view, nothing to do
         ERLogging::write_log( "Business view $delete_view not found.\n" );       
      }                       
      else {             
       # Now delete views from database
       ERLogging::write_log( "Calling SAHARA.SP_BVIEW Delete1\n" );
       $sql = "begin execute SAHARA.SP_BVIEW_Delete1 ? end";
       $sth = $dbh->prepare($sql);
       $sth->bind_param( 1, $vk );
       $sth->execute();
       $sth->finish;
      }
        # Cleanup database locks
       cleanup_db();
      
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
      $start_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d",
                          $year+1900,$mon+1,$mday,$hour,$min,$sec );

      ERLogging::write_log("\n$start_time: Done deleting views\n");
      exit( 0 );
   };

   if ( $@ ) {
      ERLogging::write_log( "Delete failed: $@\n" );
      eval { $dbh->rollback() };
   }
   $dbh->commit();
   if( $dbh ) {
                cleanup_db();
                $dbh->disconnect();
   }
   ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   $start_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d",
                          $year+1900,$mon+1,$mday,$hour,$min,$sec );

      ERLogging::write_log("\n$start_time: Done deleting views\n");
   exit 0;
}

#
# perform all of our database operations within this eval block. This ensures
# any errors are captured in the following $@ block so we can rollback on error.
# Without this, we'd have to check the return code of every db command we make.
# This also ensures we don't commit partial views when some other error occurs
# during the import process. e.g. a 'file not found' error.
#
my $success = 0;
my $viewsetname = "";
eval {
   # open a connection to the database
   init_db();
   cleanup_staging_tables();

   if( ! opendir( DEFS, $view_dir ) ) {
      ERLogging::write_log( "\nCannot open directory '$view_dir' for reading.\n\n" );
      ERLogging::write_log( "Verify this directory exists and contains business view files.\n\n" );
                  cleanup_db();
      exit 1;
   }

  # $view_def_file contains complete view file name, for example Geography.view
  # The file name is case sensitive  
   if( $view_def_file ) {
      # Search for the view definition file that user specified
      @view_defs = grep{ /^$view_def_file/i } readdir( DEFS );

      if( scalar(@view_defs) == 0 ) {
         ERLogging::write_log( "\n'$view_def_file' was not found in '$view_dir'.\n" );
         ERLogging::write_log( "\nVerify this file exists and is readable by the current user.\n\n" );
                                 cleanup_db();
         exit 1;
      }

   }
   else {
      # User has not specified a file, so find all view definitions 
      # in the directory specified
      @view_defs = grep { /\.view$/i } readdir( DEFS );
   }
   closedir( DEFS );

   if( scalar(@view_defs) == 0 ) {
      ERLogging::write_log( "\nNo '.view' files were detected in directory '$view_dir'.\n\n" );
      ERLogging::write_log( "Verify this directory exists and contains business view files.\n" );
                  cleanup_db();
      exit 1;
   }

   ERLogging::write_log( "-" x 79 );
   ERLogging::write_log( "\nStarting new import session at " . $start_time . "\n" );
   ERLogging::write_log( "Found the following business view files:\n" );
   for( my $i = 0; $i < @view_defs; ++$i ) {
      ERLogging::write_log("    $view_defs[ $i ]\n" );
      $view_defs[ $i ] =~ s/\.view\z//;
   }

   foreach $viewsetname ( @view_defs ) {
      # open a connection to the database
      #init_db();
      cleanup_staging_tables();

      # reset after the previous view
      @levels = ();
      @levelDescriptions = ();
      @types = ();
      @reject_files = ();
      @sqlldr_log_files = ();
      @view_assignment_files = ();

      # if we can't read the def file for this view, move on to the next view.
      if( !read_def_file( $viewsetname ) ) {
         set_status( $viewsetname, "FAILED" );
         next;
      }

      ERLogging::write_log( "\n" );
      ERLogging::write_log( "-" x 79 );
      ERLogging::write_log( "\nFound view set \'" . $viewsetname . "\' - processing business view.\n" );
      print "\n\n";
      my $skip = 0;
      eval {

         ERLogging::write_log( "Populating staging tables\n" );

         ERLogging::write_log( "Populating overview staging table\n" );
         populate_view_overview_staging_table( $viewsetname );

         ERLogging::write_log( "Populating level staging table\n" );
         populate_level_staging_table( $viewsetname );

         ERLogging::write_log( "Populating type staging table\n" );
         populate_types_staging_table( $viewsetname );

         ERLogging::write_log( "Populating node staging table\n" );
         populate_node_staging_table( $viewsetname );

         ERLogging::write_log( "Populating view assignment staging table\n" );
         populate_view_assignment_staging_tables( $viewsetname );

         add_log_message( $viewsetname, "Business view successfully staged." );

         # correlate to existing views and levels
         if( ! $assignment_only ) {
            ERLogging::write_log( "Correlating view and view levels\n" );
           #  $skip = correlate_view();
            if( ! correlate_view() ) {
               $skip = 1;
            }
         }

         if( ! $skip ) {
            ERLogging::write_log( "Correlating nodes and objects\n" );
   my $sql = "begin execute SAHARA_APP.SP_IE_CorrelateNodesAndObjects ?, ? end";
   my $hdl = $dbh->prepare($sql);
    $hdl->bind_param(1, $view_oracle_name);
    $hdl->bind_param(2, $session_key);
    $hdl->execute();
    $hdl->finish;
   #         correlate_nodes_and_objects();

            ERLogging::write_log( "Saving reject records to the .rej file\n" );
            save_rejected_records( $viewsetname );

            get_record_counts( $viewsetname );
         }
      };

      if ( $@ ) {
         my $err = $DBI::err || 0;
         my $msg = $DBI::errstr || "";
            ERLogging::write_log( "Error code: $err and errormessage: $msg\n" );

         if( $err == 20917 ) {
            add_log_message( $viewsetname, "Business view exists and overwrite is disabled... Skipping\n" );
            ERLogging::write_log( "View \'$view_display_name\' exists and overwrite is disabled. Skipping...\n", 1 );
            set_status( $viewsetname, "SKIPPED(duplicate)" );
         }
         elsif( $err ) {
            add_log_message( $viewsetname, "Error: " . SybaseHelpers::sybase_error_message( "", $err, $msg ) . "\n" );
            ERLogging::write_log( "Error: " . SybaseHelpers::sybase_error_message( "", $err, $msg ) . "\n" );
            set_status( $viewsetname, "FAILED" );
         }
         else {
            ERLogging::write_log( "Error: $@\n" );
            set_status( $viewsetname, "$@" );
         }

         $dbh->rollback();
      }
      else {

         if( ! $skip ) {
            add_log_message( $viewsetname, "View correlated to production tables." );
            set_status( $viewsetname, "SUCCESS" );
         }
         else {
            add_log_message( $viewsetname, "View publish failed. Skipped..." );
            set_status( $viewsetname, "FAILED" );
         }
         $dbh->commit();
      }

      write_summary_report( $viewsetname, $option_file );

      # cleanup session and session locks.
      cleanup_db();

      # if any view succeeds we want to update the metamodel.
      if( !$assignment_only && $run_update_script ) {
         ERLogging::write_log("\nExecuting the update_metamodel.pl script\n");
         my $iscript = "";
         $^O eq 'MSWin32' ? ($iscript .= "perl.exe ") : ($iscript .= "perl ");

         $iscript .= "update_metamodel.pl -l $log_dir -v \"$view_display_name\"";

         open(README, "$iscript |") or die "Can't execute script: $!\n";
         while(<README>) {
             ERLogging::write_log($_);
         }
         close(README);
      }
   }
};

if ( $@ ) {
         my $err1 = $DBI::err || 0;
         my $msg1 = $DBI::errstr || "";
            ERLogging::write_log( "Error code: $err1 and errormessage: $msg1\n" );
   ERLogging::write_log( "Import failed: $@\n" );
   eval { $dbh->rollback() };

   # failure
   set_status( $viewsetname, "FAILED" );
   cleanup_db();

   # add other application on-error-clean-up code here
}

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
my $end_time = sprintf(
        "%4d-%02d-%02d %02d:%02d:%02d\n",
        $year+1900,$mon+1,$mday,$hour,$min,$sec );

if( $dbh ) {
   $dbh->disconnect();
}

ERLogging::write_log( "\n" );
ERLogging::write_log( "-" x 79 );

# print out metrics for this session.
ERLogging::write_log( "\n\n\nImport Session Metrics\n" );
ERLogging::write_log( "Total view sets found: " . scalar( @view_defs ) . "\n\n" );

ERLogging::write_log( "     View set         Object          Total     Total  Assigned  Unknown  Invalid\n" );
ERLogging::write_log( "       Name            Type            Rows   Objects   Objects  Objects     Rows \n" );
ERLogging::write_log( "----------------+----------------+---------+---------+---------+--------+---------\n" );

foreach my $viewset ( @view_defs ) {

   next if(!defined $view_types{ $viewset });
   
   my @vt = @{ $view_types{ $viewset } };

   foreach my $type ( sort @vt ) {
      my $objects = $view_object_counts{ $viewset }{ $type } || 0;
      my $assigned = $view_assigned_counts{ $viewset }{ $type } || 0;
      my $invalid = $view_invalid_counts{ $viewset }{ $type } || 0;
      my $unknown = $view_unknown_counts{ $viewset }{ $type } || 0;
      my $total = $view_row_counts{ $viewset }{ $type } || 0;

      my $tmp = sprintf("%-17.17s %#15.15s  %#8.8s  %8.8s  %#8.8s %#8.8s %#8.8s\n",
                        $viewset, $type, $total, $objects, $assigned, $unknown, $invalid  );
      ERLogging::write_log( $tmp );
   }
}

print "\n";

ERLogging::write_log( "\n\nImport session ended at " . $end_time . "\n" );
ERLogging::write_log( "\n" );
